from .db import database, engine, Base, DbSession
from .users import add_user_routers, User
from fastapi import FastAPI
from .routes import create_api_router
from starlette.requests import Request


def get_app(database) -> FastAPI:
    application = FastAPI(title="swimtracker", debug=True, version="0.1")
    fastapi_users = add_user_routers(application, database)
    current_user = fastapi_users.current_user(active=True, verified=True)
    current_superuser = fastapi_users.current_user(active=True, superuser=True, verified=True)
    application.include_router(create_api_router(current_user))
    return application


app = get_app(database)


@app.middleware("http")
async def db_session_middleware(request: Request, call_next):
    request.state.db = DbSession()
    response = await call_next(request)
    request.state.db.close()
    return response


@app.on_event("startup")
async def startup() -> None:
    print("creating")
    Base.metadata.create_all(engine)
    if not database.is_connected:
        await database.connect()


@app.on_event("shutdown")
async def shutdown() -> None:
    if database.is_connected:
        await database.disconnect()
